package com.xy.annotation.aspet;

import com.xy.annotation.CheckSysId;
import com.xy.annotation.IsAdmin;
import com.xy.error.CommRuntimeException;
import com.xy.redis.RedisCache;
import com.xy.utils.Reflect;
import com.xy.utils.Tuple;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

@Aspect
@Component
public class CommAset {

    @Around("@annotation(checkSysId)")
    public Object checkSysId(ProceedingJoinPoint joinPoint, CheckSysId checkSysId) throws Throwable {
        Object[] args = joinPoint.getArgs();
        Object result;
        String value = checkSysId.value();
        try {
            String[] split = value.split(",");
            for (String s : split) {
                Tuple.Tuple3<String, Object, Class<?>> property = Reflect.PropertyReflect.getProperty(args[0], s);
                if (property.getV2() != null) {
                    List list = property.getV3().equals(List.class) ? (List) property.getV2() : Arrays.asList(property.getV2());
                    list.forEach(o -> RedisCache.checkSystemIdByException(Long.parseLong(o.toString())));
                    break;
                }
            }
            result = joinPoint.proceed(args);
        } catch (Exception e) {
            throw e;
        }
        return result;
    }

    @Around("@annotation(isAdmin)")
    public Object isAdmin(ProceedingJoinPoint joinPoint, IsAdmin isAdmin) throws Throwable {
        Object[] args = joinPoint.getArgs();
        Object result;
        try {
            if (!RedisCache.isAdmin()) {
                throw new CommRuntimeException("权限不足");
            }
            result = joinPoint.proceed(args);
        } catch (Exception e) {
            throw e;
        }
        return result;
    }
}
