package com.xy.config;

import cn.dev33.satoken.stp.StpUtil;
import com.xy.ServerWebConfig;
import com.xy.enums.SaTokenEnum;
import com.xy.error.CommRuntimeException;
import com.xy.swagger.Knife4jConfig;
import com.xy.utils.Emptys;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 权限系统权限拦截器
 */
@Component
@AllArgsConstructor
public class SaTokenInterceptorImpl extends HandlerInterceptorAdapter implements WebMvcConfigurer {

    private SaTokenAuthorizeConfig saTokenAuthorizeConfig;

    /**
     * 方法执行前
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        String satoken = request.getHeader(saTokenAuthorizeConfig.getTokenName());
        if (!Emptys.check(satoken)) {
            throw new CommRuntimeException(SaTokenEnum.NO_LOGIN.getKey(), SaTokenEnum.NO_LOGIN.getName());
        }
        Object loginIdByToken = StpUtil.getLoginIdByToken(satoken);
        if (loginIdByToken == null) {
            throw new CommRuntimeException(SaTokenEnum.NO_LOGIN.getKey(), SaTokenEnum.NO_LOGIN.getName());
        }
        return true;
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        List<String> excludePaths = saTokenAuthorizeConfig.getExcludePaths();
        excludePaths.addAll(Knife4jConfig.swaggerExcludes);
        excludePaths.addAll(ServerWebConfig.serverWebExcludes);
        registry.addInterceptor(this)
                .excludePathPatterns(excludePaths)
                .addPathPatterns("/**");
    }

}
