package com.xy.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.annotation.CheckSysId;
import com.xy.annotation.IsAdmin;
import com.xy.collections.list.JArrayList;
import com.xy.collections.list.JList;
import com.xy.consts.CommConsts;
import com.xy.dto.SysMenuDto;
import com.xy.entity.SysMenu;
import com.xy.entity.SysRole;
import com.xy.entity.SysRoleMenu;
import com.xy.entity.join.SysMenuJoinSysRoleMenu;
import com.xy.mapper.SysMenuMapper;
import com.xy.mapper.SysRoleMapper;
import com.xy.mapper.SysRoleMenuMapper;
import com.xy.redis.RedisCache;
import com.xy.service.SysMenuService;
import com.xy.service.SysRoleMenuService;
import com.xy.utils.R;
import com.xy.utils.SaTokenUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.Beans.tree;

/**
 * <p>
 * 菜单表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2022-12-09
 */
@Service
@AllArgsConstructor
@Api(tags = "菜单接口")
public class SysMenuServiceImpl extends ServiceImpl<SysMenuMapper, SysMenu> implements SysMenuService {

    private SysRoleMenuMapper sysRoleMenuMapper;

    @Override
    @CheckSysId
    @ApiOperation("用户菜单树")
    public R<List<SysMenuDto.Vo>> workMenu(SysMenuDto.WorkMenu workMenu) {
        List<SysMenuDto.Vo> list = new ArrayList<>();
        Long longId = SaTokenUtils.getId(Long.class);
        List<SysMenuJoinSysRoleMenu> menus = RedisCache.getMenuTree(longId, workMenu.getSystemId());
        menus.forEach(sysMenuJoinSysRoleMenu -> {
            SysMenuDto.Vo vo = copy(SysMenuDto.Vo.class, sysMenuJoinSysRoleMenu);
            List<SysMenuDto.Vo> sonVos = copy(SysMenuDto.Vo.class, sysMenuJoinSysRoleMenu.getSonSysMenu());
            list.add(vo.setSonSysMenu(sonVos));
        });
        return R.ok(list);
    }

    @IsAdmin
    @Override
    @ApiOperation("全部菜单树")
    public R<List<SysMenuDto.Vo>> menu(SysMenuDto.WorkMenu workMenu) {
        List<SysMenuDto.Vo> vos = copy(SysMenuDto.Vo.class, list());
        JList<SysMenuDto.Vo> list = new JArrayList<>(vos).filter().isNull(SysMenuDto.Vo::getPaterId).list();
        for (SysMenuDto.Vo vo : list) {
            List<SysMenuDto.Vo> tree = tree(vo, vos, SysMenuDto.Vo::getId, SysMenuDto.Vo::getPaterId, SysMenuDto.Vo::getSonSysMenu);
            vo.setSonSysMenu(tree);
        }
        return R.ok(list);
    }

    @IsAdmin
    @Override
    @ApiOperation("添加菜单")
    public R save(SysMenuDto.Save save) {
        //校验重复
        Long longId = SaTokenUtils.getId(Long.class);
        long count = count(new LambdaQueryWrapper<SysMenu>()
                .eq(SysMenu::getCode, save.getCode())
                .eq(SysMenu::getSysId, save.getSysId())
        );
        if (count > 0) {
            return R.fail("标识已存在");
        }
        //添加菜单
        SysMenu sysMenu = copy(SysMenu.class, save)
                .create(longId);
        save(sysMenu);
        //角色赋予管理员
        SysRoleMenu sysRoleMenu = new SysRoleMenu()
                .createId()
                .setMenuId(sysMenu.getId())
                .setRoleId(CommConsts.ADMIN_ROLE_ID)
                .setSelectIsAdmin(true);
        sysRoleMenuMapper.insert(sysRoleMenu);
        return R.ok();
    }

    @IsAdmin
    @Override
    @ApiOperation("修改菜单")
    public R update(SysMenuDto.Update update) {
        Long longId = SaTokenUtils.getId(Long.class);
        SysMenu sysMenu = copy(SysMenu.class, update)
                .update(longId);
        updateById(sysMenu);
        return R.ok();
    }
}
