package com.xy.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.BatchDeleteDto;
import com.xy.dto.SysRoleDto;
import com.xy.entity.SysRole;
import com.xy.mapper.SysRoleMapper;
import com.xy.service.SysRoleMenuService;
import com.xy.service.SysRoleService;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import com.xy.utils.SaTokenUtils;
import com.xy.vo.RoleMenuVo;
import com.xy.vo.SysRoleVo;
import io.swagger.annotations.Api;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;

/***
 * 角色服务实现类
 * @author 谭斌
 * @date 2022/12/12 17:17
 */
@Api(tags = "角色管理")
@Service
@RequiredArgsConstructor
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {

    private final SysRoleMenuService sysRoleMenuService;

    @Override
    public R<Boolean> saveOrUpdate(SysRoleDto sysRoleDto) {
        Long id = sysRoleDto.getId();
        long currentUser = Long.parseLong(SaTokenUtils.getId().toString());
        LambdaQueryWrapper<SysRole> lqw = new LambdaQueryWrapper<SysRole>()
                .eq(SysRole::getCode, sysRoleDto.getCode())
                .eq(SysRole::getSysId, sysRoleDto.getSysId());
        if (id != null) {
            lqw.ne(SysRole::getId, id);
        }

        long count = count(lqw);
        if (count > 0) {
            return R.fail("此角色标识已存在");
        }

        SysRole sysRole = copy(SysRole.class, sysRoleDto).saveOrUpdate(currentUser, sysRoleDto.getSysId());

        return R.ok(saveOrUpdate(sysRole));
    }

    @Override
    public R<Boolean> batchDelete(BatchDeleteDto batchDeleteDto) {
        return R.ok(removeByIds(batchDeleteDto.getIds()));
    }

    @Override
    public R<SysRoleVo> getById(Long id) {
        SysRole role = super.getById(id);
        SysRoleVo sysRoleVo = copy(SysRoleVo.class, role);
        //角色菜单
        sysRoleVo.setMenuIds(sysRoleMenuService.getRoleMenus(id));
        return R.ok(sysRoleVo);
    }

    @Override
    public R<PageBean<SysRoleVo>> getRolePage(PageBean pageBean, SysRoleDto sysRoleDto) {
        IPage<SysRole> page = page(toIPage(pageBean), Wrappers.<SysRole>lambdaQuery()
                .like(StrUtil.isNotBlank(sysRoleDto.getName()), SysRole::getName, sysRoleDto.getName())
                .like(StrUtil.isNotBlank(sysRoleDto.getCode()), SysRole::getCode, sysRoleDto.getCode()));
        return R.ok(toPageBean(SysRoleVo.class, page));

    }

    @Override
    public R<Boolean> saveRoleMenus(RoleMenuVo roleMenuVo) {
        return R.ok(sysRoleMenuService.saveRoleMenus(roleMenuVo.getId(), roleMenuVo.getMenuIds()));
    }


}
