package com.xy.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.annotation.CheckSysId;
import com.xy.annotation.IsAdmin;
import com.xy.dto.SysSystemDto;
import com.xy.entity.SysSystem;
import com.xy.mapper.SysSystemMapper;
import com.xy.redis.RedisCache;
import com.xy.service.SysSystemService;
import com.xy.utils.MybatisPlusQuery;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import com.xy.utils.SaTokenUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;

/**
 * <p>
 * 系统表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2022-12-09
 */
@Service
@Api(tags = "系统接口")
@AllArgsConstructor
public class SysSystemServiceImpl extends ServiceImpl<SysSystemMapper, SysSystem> implements SysSystemService {

    @IsAdmin
    @PostMapping("save")
    @ApiOperation("添加系统")
    public R<Long> save(@RequestBody @Validated SysSystemDto.Save save) {
        long count = count(new LambdaQueryWrapper<SysSystem>().eq(SysSystem::getCode, save.getCode()));
        if (count > 0) {
            return R.fail("标识已存在");
        }
        long loginId = Long.parseLong(SaTokenUtils.getId().toString());
        SysSystem sysSystem = copy(SysSystem.class, save)
                .create(loginId);
        save(sysSystem);
        return R.ok(sysSystem.getId());
    }

    @IsAdmin
    @PostMapping("update")
    @ApiOperation("修改系统")
    public R update(@RequestBody @Validated SysSystemDto.Update update) {
        SysSystem sysSystem = copy(SysSystem.class, update).update(Long.parseLong(SaTokenUtils.getId().toString()));
        updateById(sysSystem);
        return R.ok();
    }

    @PostMapping("list")
    @ApiOperation("登录人所属系统列表")
    public R<List<SysSystemDto.Vo>> list(@RequestBody SysSystemDto.Select select) {
        List<Long> systemIds = RedisCache.getSystemIds(SaTokenUtils.getId(Long.class));
        LambdaQueryWrapper<SysSystem> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(select, SysSystem.class)
                .in(SysSystem::getId, systemIds)
                .ge(SysSystem::getCreateTime, select.getBeginCreateTime())
                .le(SysSystem::getCreateTime, select.getEndCreateTime())
                .like(SysSystem::getName)
                .build();
        List<SysSystem> list = list(lambdaQueryWrapper);
        return R.ok(copy(SysSystemDto.Vo.class, list));
    }

    @IsAdmin
    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<SysSystemDto.Vo>> page(@RequestBody @Validated SysSystemDto.PageSelect pageSelect) {
        LambdaQueryWrapper<SysSystem> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(pageSelect, SysSystem.class)
                .ge(SysSystem::getCreateTime, pageSelect.getBeginCreateTime())
                .le(SysSystem::getCreateTime, pageSelect.getEndCreateTime())
                .like(SysSystem::getName)
                .build();
        //数据权限
        RedisCache.getDataAuth(SaTokenUtils.getId(Long.class), pageSelect.getSystemId(), lambdaQueryWrapper, SysSystem::getCreateUser);
        IPage<SysSystem> page = page(toIPage(pageSelect.getPage()), lambdaQueryWrapper);
        return R.ok(toPageBean(SysSystemDto.Vo.class, page));
    }
}
