package com.xy.service;

import cn.hutool.core.lang.tree.Tree;
import com.xy.annotate.RestMappingController;
import com.xy.dto.SysDeptDto;
import com.xy.utils.R;
import com.xy.vo.SysDeptVo;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/***
 * 部门管理服务类
 * @author 谭斌
 * @date 2022/12/12 16:37
 */
@RestMappingController("sysDept")
public interface SysDeptService {

    @ApiOperation(value = "新增、更新部门信息", notes = "新增、更新部门信息")
    @PostMapping("saveOrUpdate")
    R<Boolean> saveOrUpdate(@Valid @RequestBody SysDeptDto sysDeptDto);

    @ApiOperation(value = "系统部门树", notes = "系统部门树")
    @GetMapping("tree/{sysId}")
    R<List<Tree<Long>>> deptTree(@PathVariable("sysId") Long sysId);

    @ApiOperation(value = "根据ID获取部门信息", notes = "根据ID获取部门信息")
    @GetMapping("{id}")
    R<SysDeptVo> getById(@PathVariable("id") Long id);


    @ApiOperation(value = "删除部门", notes = "删除部门")
    @DeleteMapping("{id}")
    R<Boolean> deleteById(@PathVariable("id") Long id);

    @ApiOperation(value = "获取指定部门部门树", notes = "获取指定部门部门树")
    @GetMapping("deptTree/{id}")
    R<List<Tree<Long>>> getDeptTreeById(@PathVariable("id") Long id);

}
